@@ -29,6 +29,7 @@ def data_threshold(data_type: str) -> int:
 
 
 class Connector:
+    __initial_all_data_received = False
     __last_time_stamp: float = -1
     __last_sub_time: float = 0
     __record_data: bool = False
@@ -88,6 +89,7 @@ class Connector:
     on_room_left: Union[Callable[[DeviceLeftMsg], None], Callable[[DeviceLeftMsg, Connector], None]] = noop
 
     on_sprite_out: Union[Callable[[SpriteOutMsg], None], Callable[[SpriteOutMsg, Connector], None]] = noop
+    on_sprite_removed: Union[Callable[[SpriteRemovedMsg], None], Callable[[SpriteRemovedMsg, Connector], None]] = noop
     on_sprite_collision: Union[Callable[[SpriteCollisionMsg], None],
                                Callable[[SpriteCollisionMsg, Connector], None]] = noop
     on_overlap_in: Union[Callable[[SpriteCollisionMsg], None],
@@ -137,11 +139,11 @@ class Connector:
 
         if the sprite is not found, None is returned
         '''
-        if len(self.__sprites) == 0:
+        if len(self.sprites) == 0:
             return None
         if id is None:
-            return self.__sprites[0]
-        return first(lambda s: s.id == id, self.__sprites)
+            return self.sprites[0]
+        return first(lambda s: s.id == id, self.sprites)
 
     get_circle = get_sprite
     get_ellipse = get_sprite
@@ -723,7 +725,7 @@ class Connector:
         sprites = []
 
         def sprite(
-                id: Optional[str],
+                id: Optional[str] = None,
                 clickable: Optional[bool] = None,
                 collision_detection: Optional[bool] = None,
                 color: Optional[Union[Colors, str]] = None,
@@ -857,7 +859,7 @@ class Connector:
             raise
         else:
             for s in sprites:
-                to_update = first(lambda s: s['id'] == s['id'], self.__sprites)
+                to_update = first(lambda spr: spr['id'] == s['id'], self.__sprites)
                 if to_update is not None:
                     to_update.update(s)
                 else:
@@ -1489,7 +1491,7 @@ class Connector:
         return sprite['id']
 
     def clear_playground(self, **delivery_opts):
-        self.__sprites.clear()
+        self.__sprites = []
         self.emit(
             SocketEvents.NEW_DATA,
             {
@@ -1703,12 +1705,12 @@ class Connector:
             lines = []
             raise
         else:
-            for s in lines:
-                to_update = first(lambda s: s['id'] == s['id'], self.__lines)
+            for l in lines:
+                to_update = first(lambda ln: ln['id'] == l['id'], self.__lines)
                 if to_update is not None:
-                    to_update.update(s)
+                    to_update.update(l)
                 else:
-                    self.__lines.append(DictX(s))
+                    self.__lines.append(DictX(l))
             self.emit(
                 SocketEvents.NEW_DATA,
                 {
@@ -2287,12 +2289,37 @@ class Connector:
             elif data['type'] == DataType.ALERT_CONFIRM:
                 self.__alerts.append(cast(AlertConfirmMsg, data))
             elif data['type'] == DataType.SPRITE_OUT:
-                sprite = first(lambda s: s.id == data.sprite_id, self.__sprites)
-                if sprite:
-                    self.__sprites.remove(sprite)
+                sprite = self.get_sprite(data['id'])
+                if sprite is not None:
+                    data.update({
+                        'sprite': deepcopy(sprite)
+                    })
                 self.__callback('on_sprite_out', data)
+            elif data['type'] == DataType.SPRITE_REMOVED:
+                sprite = self.get_sprite(data['id'])
+                if sprite is not None:
+                    data.update({'sprite': deepcopy(sprite)})
+                    self.__sprites.remove(sprite)
+                self.__callback('on_sprite_removed', data)
             elif data['type'] == DataType.SPRITE_COLLISION:
-                data['sprites'] = list(map(lambda s: DictX(s), data['sprites']))
+                if len(data['sprites']) == 2:
+                    try:
+                        s1 = self.get_sprite(data['sprites'][0]['id'])
+                        s2 = self.get_sprite(data['sprites'][0]['id'])
+                        if s1 is not None:
+                            data['sprites'][0] = deepcopy(s1)
+                        else:
+                            data['sprites'][0] = DictX(data['sprites'][0])
+
+                        if s2 is not None:
+                            data['sprites'][1] = deepcopy(s2)
+                        else:
+                            data['sprites'][1] = DictX(data['sprites'][1])
+                    except:
+                        data['sprites'] = list(map(lambda s: DictX(s), data['sprites']))
+                else:
+                    data['sprites'] = list(map(lambda s: DictX(s), data['sprites']))
+
                 self.__callback('on_sprite_collision', data)
                 if data['overlap'] == 'in':
                     self.__callback('on_overlap_in', data)
@@ -2300,6 +2327,9 @@ class Connector:
                     self.__callback('on_overlap_out', data)
 
             elif data['type'] == DataType.BORDER_OVERLAP:
+                original = self.get_sprite(data['id'])
+                if original is not None:
+                    data.update({'sprite': deepcopy(original)})
                 self.__callback('on_border_overlap', data)
             elif data['type'] == DataType.SPRITE_CLICKED:
                 self.__callback('on_sprite_clicked', data)
@@ -2313,16 +2343,21 @@ class Connector:
     def __on_all_data(self, data: dict):
         if 'device_id' not in data:
             return
-
         xdata: dict[str, List[dict]] = data['all_data']
         for dtype in xdata:
             xdata[dtype] = list(map(lambda msg: DictX(msg), xdata[dtype]))
 
         data['all_data'] = DictX(xdata)
         self.data[data['device_id']] = data['all_data']
-        if DataType.SPRITE in data['all_data'] and data['device_id'] == self.device_id:
-            self.__sprites = list(map(lambda s: s['sprite'], data['all_data']['sprite']))
+        # if DataType.SPRITES in data['all_data'] and data['device_id'] == self.device_id:
+        #     if self.__initial_all_data_received:
+        #         self.__sprites = list(map(lambda s: s['sprite'], data['all_data']['sprites']))
+        #     else:
+        #         for s in data['all_data']['sprites']:
+        #             if 'id' in s and self.get_sprite(s['id']) is None:
+        #                 self.__sprites.append(DictX(s))
 
+        self.__initial_all_data_received = True
         self.__callback('on_all_data', data)
 
     def __on_room_left(self, device: dict):
